@@ -56,6 +56,7 @@ extensions = [
     "sphinx_autosummary_accessors",
     "sphinx_copybutton",
     "sphinx_design",
+    "sphinx_favicon",
 ]
 
 # Add any paths that contain templates here, relative to this directory.
@@ -64,7 +65,7 @@ templates_path = ["_templates", sphinx_autosummary_accessors.templates_path]
 # List of patterns, relative to source directory, that match files and
 # directories to ignore when looking for source files.
 # This pattern also affects html_static_path and html_extra_path.
-exclude_patterns = []
+exclude_patterns = ["Thumbs.db", ".DS_Store"]
 
 
 # -- Options for HTML output -------------------------------------------------
@@ -114,24 +115,25 @@ html_theme_options = {
             "icon": "fa-brands fa-twitter",
         },
     ],
-    "favicons": [
-        {
-            "rel": "icon",
-            "sizes": "32x32",
-            "href": "https://raw.githubusercontent.com/pola-rs/polars-static/master/icons/favicon-32x32.png",
-        },
-        {
-            "rel": "apple-touch-icon",
-            "sizes": "180x180",
-            "href": "https://raw.githubusercontent.com/pola-rs/polars-static/master/icons/touchicon-180x180.png",
-        },
-    ],
     "logo": {
         "image_light": "https://raw.githubusercontent.com/pola-rs/polars-static/master/logos/polars-logo-dark-medium.png",
         "image_dark": "https://raw.githubusercontent.com/pola-rs/polars-static/master/logos/polars-logo-dimmed-medium.png",
     },
 }
 
+favicons = [
+    {
+        "rel": "icon",
+        "sizes": "32x32",
+        "href": "https://raw.githubusercontent.com/pola-rs/polars-static/master/icons/favicon-32x32.png",
+    },
+    {
+        "rel": "apple-touch-icon",
+        "sizes": "180x180",
+        "href": "https://raw.githubusercontent.com/pola-rs/polars-static/master/icons/touchicon-180x180.png",
+    },
+]
+
 intersphinx_mapping = {
     "numpy": ("https://numpy.org/doc/stable/", None),
     "pandas": ("https://pandas.pydata.org/docs/", None),
@@ -201,7 +203,7 @@ def linkcode_resolve(domain, info):
 
 def _minify_classpaths(s: str) -> str:
     # strip private polars classpaths, leaving the classname:
-    # * "pli.Expr" -> "Expr"
+    # * "pl.Expr" -> "Expr"
     # * "polars.expr.expr.Expr" -> "Expr"
     # * "polars.lazyframe.frame.LazyFrame" -> "LazyFrame"
     # also:
@@ -211,9 +213,9 @@ def _minify_classpaths(s: str) -> str:
         pattern=r"""
         ~?
         (
-          (?:pli|
+          (?:pl|
             (?:polars\.
-              (?:internals|datatypes)
+              (?:_reexport|datatypes)
             )
           )
           (?:\.[a-z.]+)?\.
