@@ -1,4 +1,7 @@
+pub mod bitmap;
 mod boolean;
+#[cfg(feature = "dtype-array")]
+mod fixed_size_list;
 
 use arrow::array::*;
 use arrow::bitmap::MutableBitmap;
@@ -9,12 +12,15 @@ use arrow::types::NativeType;
 
 use crate::bit_util::unset_bit_raw;
 use crate::prelude::*;
-use crate::trusted_len::{PushUnchecked, TrustedLen};
+use crate::trusted_len::{TrustedLen, TrustedLenPush};
 use crate::utils::{with_match_primitive_type, CustomIterTools};
 
 /// # Safety
 /// Does not do bounds checks
 pub unsafe fn take_unchecked(arr: &dyn Array, idx: &IdxArr) -> ArrayRef {
+    if idx.null_count() == idx.len() {
+        return new_null_array(arr.data_type().clone(), idx.len());
+    }
     use PhysicalType::*;
     match arr.data_type().to_physical_type() {
         Primitive(primitive) => with_match_primitive_type!(primitive, |$T| {
@@ -33,6 +39,11 @@ pub unsafe fn take_unchecked(arr: &dyn Array, idx: &IdxArr) -> ArrayRef {
             let arr = arr.as_any().downcast_ref().unwrap();
             Box::new(boolean::take_unchecked(arr, idx))
         }
+        #[cfg(feature = "dtype-array")]
+        FixedSizeList => {
+            let arr = arr.as_any().downcast_ref().unwrap();
+            Box::new(fixed_size_list::take_unchecked(arr, idx))
+        }
         // TODO! implement proper unchecked version
         #[cfg(feature = "compute")]
         _ => {
