@@ -1,5 +1,7 @@
 use std::fmt::Debug;
 
+use num_traits::ToPrimitive;
+
 use super::*;
 use crate::index::IdxSize;
 use crate::trusted_len::TrustedLen;
@@ -16,8 +18,8 @@ where
     T: std::iter::Sum<T>
         + NativeType
         + Copy
-        + std::cmp::PartialOrd
-        + num::ToPrimitive
+        + PartialOrd
+        + ToPrimitive
         + NumCast
         + Add<Output = T>
         + Sub<Output = T>
@@ -27,11 +29,7 @@ where
 {
     if values.is_empty() {
         let out: Vec<T> = vec![];
-        return Box::new(PrimitiveArray::from_data(
-            T::PRIMITIVE.into(),
-            out.into(),
-            None,
-        ));
+        return Box::new(PrimitiveArray::new(T::PRIMITIVE.into(), out.into(), None));
     }
 
     let mut sorted_window = SortedBuf::new(values, 0, 1);
@@ -62,8 +60,8 @@ pub(crate) fn compute_quantile2<T>(
 where
     T: std::iter::Sum<T>
         + Copy
-        + std::cmp::PartialOrd
-        + num::ToPrimitive
+        + PartialOrd
+        + ToPrimitive
         + NumCast
         + Add<Output = T>
         + Sub<Output = T>
@@ -126,12 +124,13 @@ pub fn rolling_median<T>(
     min_periods: usize,
     center: bool,
     weights: Option<&[f64]>,
-) -> ArrayRef
+    _params: DynArgs,
+) -> PolarsResult<ArrayRef>
 where
     T: NativeType
         + std::iter::Sum<T>
-        + std::cmp::PartialOrd
-        + num::ToPrimitive
+        + PartialOrd
+        + ToPrimitive
         + NumCast
         + Add<Output = T>
         + Sub<Output = T>
@@ -140,7 +139,7 @@ where
         + Zero
         + IsFloat,
 {
-    rolling_quantile(
+    Ok(rolling_quantile(
         values,
         0.5,
         QuantileInterpolOptions::Linear,
@@ -148,7 +147,7 @@ where
         min_periods,
         center,
         weights,
-    )
+    ))
 }
 
 pub fn rolling_quantile<T>(
@@ -163,8 +162,8 @@ pub fn rolling_quantile<T>(
 where
     T: NativeType
         + std::iter::Sum<T>
-        + std::cmp::PartialOrd
-        + num::ToPrimitive
+        + PartialOrd
+        + ToPrimitive
         + NumCast
         + Add<Output = T>
         + Sub<Output = T>
@@ -245,7 +244,7 @@ where
         .collect_trusted::<Vec<T>>();
 
     let validity = create_validity(min_periods, len, window_size, det_offsets_fn);
-    Box::new(PrimitiveArray::from_data(
+    Box::new(PrimitiveArray::new(
         T::PRIMITIVE.into(),
         out.into(),
         validity.map(|b| b.into()),
@@ -285,7 +284,7 @@ where
         .collect_trusted::<Vec<T>>();
 
     let validity = create_validity(min_periods, len, window_size, det_offsets_fn);
-    Box::new(PrimitiveArray::from_data(
+    Box::new(PrimitiveArray::new(
         T::PRIMITIVE.into(),
         out.into(),
         validity.map(|b| b.into()),
@@ -380,7 +379,7 @@ mod test {
         ];
 
         for interpol in interpol_options {
-            let out1 = rolling_min(values, 2, 2, false, None);
+            let out1 = rolling_min(values, 2, 2, false, None, None).unwrap();
             let out1 = out1.as_any().downcast_ref::<PrimitiveArray<f64>>().unwrap();
             let out1 = out1.into_iter().map(|v| v.copied()).collect::<Vec<_>>();
             let out2 = rolling_quantile(values, 0.0, interpol, 2, 2, false, None);
@@ -388,7 +387,7 @@ mod test {
             let out2 = out2.into_iter().map(|v| v.copied()).collect::<Vec<_>>();
             assert_eq!(out1, out2);
 
-            let out1 = rolling_max(values, 2, 2, false, None);
+            let out1 = rolling_max(values, 2, 2, false, None, None).unwrap();
             let out1 = out1.as_any().downcast_ref::<PrimitiveArray<f64>>().unwrap();
             let out1 = out1.into_iter().map(|v| v.copied()).collect::<Vec<_>>();
             let out2 = rolling_quantile(values, 1.0, interpol, 2, 2, false, None);
