@@ -1,9 +1,9 @@
+use std::fmt::{Debug, Formatter};
 use std::hash::{Hash, Hasher};
 
 use arrow::array::*;
 use hashbrown::hash_map::{Entry, RawEntryMut};
-use polars_arrow::trusted_len::PushUnchecked;
-use polars_utils::HashSingle;
+use polars_arrow::trusted_len::TrustedLenPush;
 
 use crate::datatypes::PlHashMap;
 use crate::frame::groupby::hashing::HASHMAP_INIT_SIZE;
@@ -46,7 +46,7 @@ impl RevMappingBuilder {
     }
 }
 
-#[derive(Clone, Debug)]
+#[derive(Clone)]
 pub enum RevMapping {
     /// Hashmap: maps the indexes from the global cache/categorical array to indexes in the local Utf8Array
     /// Utf8Array: caches the string values
@@ -55,6 +55,19 @@ pub enum RevMapping {
     Local(Utf8Array<i64>),
 }
 
+impl Debug for RevMapping {
+    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
+        match self {
+            RevMapping::Global(_, _, _) => {
+                write!(f, "global")
+            }
+            RevMapping::Local(_) => {
+                write!(f, "local")
+            }
+        }
+    }
+}
+
 impl Default for RevMapping {
     fn default() -> Self {
         let slice: &[Option<&str>] = &[];
@@ -75,6 +88,10 @@ impl RevMapping {
         matches!(self, Self::Global(_, _, _))
     }
 
+    pub fn is_local(&self) -> bool {
+        !self.is_global()
+    }
+
     /// Get the length of the [`RevMapping`]
     pub fn len(&self) -> usize {
         match self {
@@ -208,7 +225,7 @@ impl CategoricalChunkedBuilder<'_> {
 }
 impl<'a> CategoricalChunkedBuilder<'a> {
     fn push_impl(&mut self, s: &'a str, store_hashes: bool) {
-        let h = self.local_mapping.hasher().hash_single(s);
+        let h = self.local_mapping.hasher().hash_one(s);
         let key = StrHashLocal::new(s, h);
         let mut idx = self.local_mapping.len() as u32;
 
@@ -232,7 +249,7 @@ impl<'a> CategoricalChunkedBuilder<'a> {
 
     /// Check if this categorical already exists
     pub fn exits(&self, s: &str) -> bool {
-        let h = self.local_mapping.hasher().hash_single(s);
+        let h = self.local_mapping.hasher().hash_one(s);
         let key = StrHashLocal::new(s, h);
         self.local_mapping.contains_key(&key)
     }
