@@ -6,7 +6,7 @@ use std::ops::BitAnd;
 
 use arrow::bitmap::MutableBitmap;
 use arrow::offset::OffsetsBuffer;
-use polars_arrow::trusted_len::PushUnchecked;
+use polars_arrow::trusted_len::TrustedLenPush;
 use smartstring::alias::String as SmartString;
 
 use super::*;
@@ -36,12 +36,25 @@ fn arrays_to_fields(field_arrays: &[ArrayRef], fields: &[Series]) -> Vec<ArrowFi
         .collect()
 }
 
-fn fields_to_struct_array(fields: &[Series]) -> (ArrayRef, Vec<Series>) {
+fn fields_to_struct_array(fields: &[Series], physical: bool) -> (ArrayRef, Vec<Series>) {
     let fields = fields.iter().map(|s| s.rechunk()).collect::<Vec<_>>();
 
     let field_arrays = fields
         .iter()
-        .map(|s| s.rechunk().to_arrow(0))
+        .map(|s| {
+            let s = s.rechunk();
+            match s.dtype() {
+                #[cfg(feature = "object")]
+                DataType::Object(_) => s.to_arrow(0),
+                _ => {
+                    if physical {
+                        s.chunks()[0].clone()
+                    } else {
+                        s.to_arrow(0)
+                    }
+                }
+            }
+        })
         .collect::<Vec<_>>();
     // we determine fields from arrays as there might be object arrays
     // where the dtype is bound to that single array
@@ -118,7 +131,11 @@ impl StructChunked {
             let field_arrays = self
                 .fields
                 .iter()
-                .map(|s| s.to_arrow(i))
+                .map(|s| match s.dtype() {
+                    #[cfg(feature = "object")]
+                    DataType::Object(_) => s.to_arrow(i),
+                    _ => s.chunks()[i].clone(),
+                })
                 .collect::<Vec<_>>();
 
             // we determine fields from arrays as there might be object arrays
@@ -149,7 +166,7 @@ impl StructChunked {
                 .collect(),
         );
         let field = Field::new(name, dtype);
-        let (arrow_array, fields) = fields_to_struct_array(fields);
+        let (arrow_array, fields) = fields_to_struct_array(fields, true);
 
         let mut out = Self {
             fields,
@@ -165,23 +182,47 @@ impl StructChunked {
         let mut null_count = 0;
         let chunks_lens = self.fields()[0].chunks().len();
 
+        // fast path
+        // we early return if a column doesn't have nulls
+        for i in 0..chunks_lens {
+            for s in self.fields() {
+                let arr = &s.chunks()[i];
+                let has_nulls = arr.null_count() > 0 || matches!(s.dtype(), DataType::Null);
+                if !has_nulls {
+                    self.null_count = 0;
+                    return;
+                }
+            }
+        }
+
+        // slow path
+        // we bitand every null validity bitmask to determine
+        // in which rows all values are null
         for i in 0..chunks_lens {
-            // If all fields are null we count it as null
-            // so we bitand every chunk
             let mut validity_agg = None;
 
+            let mut all_null_array = true;
             for s in self.fields() {
                 let arr = &s.chunks()[i];
 
-                match (&validity_agg, arr.validity()) {
-                    (Some(agg), Some(validity)) => validity_agg = Some(validity.bitand(agg)),
-                    (None, Some(validity)) => validity_agg = Some(validity.clone()),
-                    _ => {}
+                if !matches!(s.dtype(), DataType::Null) {
+                    all_null_array = false;
+                    match (&validity_agg, arr.validity()) {
+                        (Some(agg), Some(validity)) => validity_agg = Some(validity.bitand(agg)),
+                        (None, Some(validity)) => validity_agg = Some(validity.clone()),
+                        _ => {}
+                    }
                 }
             }
+            // we add the null count
             if let Some(validity) = &validity_agg {
                 null_count += validity.unset_bits()
             }
+            // all arrays are null arrays
+            // we add the length of the chunk to the null_count
+            else if all_null_array {
+                null_count += self.fields()[0].chunks()[i].len()
+            }
         }
         self.null_count = null_count
     }
@@ -245,36 +286,25 @@ impl StructChunked {
     pub fn unnest(self) -> DataFrame {
         self.into()
     }
-}
 
-impl LogicalType for StructChunked {
-    fn dtype(&self) -> &DataType {
-        self.field.data_type()
+    pub(crate) fn to_arrow(&self, i: usize) -> ArrayRef {
+        let values = self
+            .fields
+            .iter()
+            .map(|s| s.to_arrow(i))
+            .collect::<Vec<_>>();
+
+        // we determine fields from arrays as there might be object arrays
+        // where the dtype is bound to that single array
+        let new_fields = arrays_to_fields(&values, &self.fields);
+        Box::new(StructArray::new(
+            ArrowDataType::Struct(new_fields),
+            values,
+            None,
+        ))
     }
 
-    /// Gets AnyValue from LogicalType
-    fn get_any_value(&self, i: usize) -> PolarsResult<AnyValue<'_>> {
-        polars_ensure!(i < self.len(), oob = i, self.len());
-        unsafe { Ok(self.get_any_value_unchecked(i)) }
-    }
-
-    unsafe fn get_any_value_unchecked(&self, i: usize) -> AnyValue<'_> {
-        let (chunk_idx, idx) = index_to_chunked_index2(&self.chunks, i);
-        if let DataType::Struct(flds) = self.dtype() {
-            // safety: we already have a single chunk and we are
-            // guarded by the type system.
-            unsafe {
-                let arr = &**self.chunks.get_unchecked(chunk_idx);
-                let arr = &*(arr as *const dyn Array as *const StructArray);
-                AnyValue::Struct(idx, arr, flds)
-            }
-        } else {
-            unreachable!()
-        }
-    }
-
-    // in case of a struct, a cast will coerce the inner types
-    fn cast(&self, dtype: &DataType) -> PolarsResult<Series> {
+    unsafe fn cast_impl(&self, dtype: &DataType, unchecked: bool) -> PolarsResult<Series> {
         match dtype {
             DataType::Struct(dtype_fields) => {
                 let map = BTreeMap::from_iter(self.fields().iter().map(|s| (s.name(), s)));
@@ -282,7 +312,13 @@ impl LogicalType for StructChunked {
                 let new_fields = dtype_fields
                     .iter()
                     .map(|new_field| match map.get(new_field.name().as_str()) {
-                        Some(s) => s.cast(&new_field.dtype),
+                        Some(s) => {
+                            if unchecked {
+                                s.cast_unchecked(&new_field.dtype)
+                            } else {
+                                s.cast(&new_field.dtype)
+                            }
+                        }
                         None => Ok(Series::full_null(
                             new_field.name(),
                             struct_len,
@@ -350,12 +386,57 @@ impl LogicalType for StructChunked {
                 let fields = self
                     .fields
                     .iter()
-                    .map(|s| s.cast(dtype))
+                    .map(|s| {
+                        if unchecked {
+                            s.cast_unchecked(dtype)
+                        } else {
+                            s.cast(dtype)
+                        }
+                    })
                     .collect::<PolarsResult<Vec<_>>>()?;
                 Ok(Self::new_unchecked(self.field.name(), &fields).into_series())
             }
         }
     }
+
+    pub(crate) unsafe fn cast_unchecked(&self, dtype: &DataType) -> PolarsResult<Series> {
+        if dtype == self.dtype() {
+            return Ok(self.clone().into_series());
+        }
+        self.cast_impl(dtype, true)
+    }
+}
+
+impl LogicalType for StructChunked {
+    fn dtype(&self) -> &DataType {
+        self.field.data_type()
+    }
+
+    /// Gets AnyValue from LogicalType
+    fn get_any_value(&self, i: usize) -> PolarsResult<AnyValue<'_>> {
+        polars_ensure!(i < self.len(), oob = i, self.len());
+        unsafe { Ok(self.get_any_value_unchecked(i)) }
+    }
+
+    unsafe fn get_any_value_unchecked(&self, i: usize) -> AnyValue<'_> {
+        let (chunk_idx, idx) = index_to_chunked_index2(&self.chunks, i);
+        if let DataType::Struct(flds) = self.dtype() {
+            // safety: we already have a single chunk and we are
+            // guarded by the type system.
+            unsafe {
+                let arr = &**self.chunks.get_unchecked(chunk_idx);
+                let arr = &*(arr as *const dyn Array as *const StructArray);
+                AnyValue::Struct(idx, arr, flds)
+            }
+        } else {
+            unreachable!()
+        }
+    }
+
+    // in case of a struct, a cast will coerce the inner types
+    fn cast(&self, dtype: &DataType) -> PolarsResult<Series> {
+        unsafe { self.cast_impl(dtype, false) }
+    }
 }
 
 #[cfg(feature = "object")]
