@@ -92,11 +92,9 @@ fn quantile_slice<T: ToPrimitive + Ord>(
     quantile: f64,
     interpol: QuantileInterpolOptions,
 ) -> PolarsResult<Option<f64>> {
-    if !(0.0..=1.0).contains(&quantile) {
-        return Err(PolarsError::ComputeError(
-            "quantile should be between 0.0 and 1.0".into(),
-        ));
-    }
+    polars_ensure!((0.0..=1.0).contains(&quantile),
+        ComputeError: "quantile should be between 0.0 and 1.0",
+    );
     if vals.is_empty() {
         return Ok(None);
     }
@@ -141,11 +139,10 @@ where
     T: PolarsNumericType,
     ChunkedArray<T>: Sortable,
 {
-    if !(0.0..=1.0).contains(&quantile) {
-        return Err(PolarsError::ComputeError(
-            "quantile should be between 0.0 and 1.0".into(),
-        ));
-    }
+    polars_ensure!(
+        (0.0..=1.0).contains(&quantile),
+        ComputeError: "`quantile` should be between 0.0 and 1.0",
+    );
 
     let null_count = ca.null_count();
     let length = ca.len();
@@ -320,6 +317,8 @@ impl Float32Chunked {
 
 impl ChunkQuantile<String> for Utf8Chunked {}
 impl ChunkQuantile<Series> for ListChunked {}
+#[cfg(feature = "dtype-array")]
+impl ChunkQuantile<Series> for ArrayChunked {}
 #[cfg(feature = "object")]
 impl<T: PolarsObject> ChunkQuantile<Series> for ObjectChunked<T> {}
 impl ChunkQuantile<bool> for BooleanChunked {}
