@@ -37,7 +37,7 @@ where
             .zip(filter.downcast_iter())
             .map(|(left, mask)| filter_fn(left, mask).unwrap())
             .collect::<Vec<_>>();
-        Ok(self.copy_with_chunks(chunks, true, true))
+        unsafe { Ok(self.copy_with_chunks(chunks, true, true)) }
     }
 }
 
@@ -58,7 +58,7 @@ impl ChunkFilter<BooleanType> for BooleanChunked {
             .zip(filter.downcast_iter())
             .map(|(left, mask)| filter_fn(left, mask).unwrap())
             .collect::<Vec<_>>();
-        Ok(self.copy_with_chunks(chunks, true, true))
+        unsafe { Ok(self.copy_with_chunks(chunks, true, true)) }
     }
 }
 
@@ -87,7 +87,7 @@ impl ChunkFilter<BinaryType> for BinaryChunked {
             .map(|(left, mask)| filter_fn(left, mask).unwrap())
             .collect::<Vec<_>>();
 
-        Ok(self.copy_with_chunks(chunks, true, true))
+        unsafe { Ok(self.copy_with_chunks(chunks, true, true)) }
     }
 }
 
