@@ -30,6 +30,11 @@ pub struct AsOfOptions {
 fn check_asof_columns(a: &Series, b: &Series, check_sorted: bool) -> PolarsResult<()> {
     let dtype_a = a.dtype();
     let dtype_b = b.dtype();
+    polars_ensure!(
+        dtype_a.to_physical().is_numeric() && dtype_b.to_physical().is_numeric(),
+        InvalidOperation:
+        "asof join only supported on numeric/temporal keys"
+    );
     polars_ensure!(
         dtype_a == dtype_b,
         ComputeError: "mismatching key dtypes in asof-join: `{}` and `{}`",
@@ -40,8 +45,8 @@ fn check_asof_columns(a: &Series, b: &Series, check_sorted: bool) -> PolarsResul
         ComputeError: "asof join must not have null values in 'on' arguments"
     );
     if check_sorted {
-        ensure_sorted_arg(a, "asof_join");
-        ensure_sorted_arg(b, "asof_join");
+        ensure_sorted_arg(a, "asof_join")?;
+        ensure_sorted_arg(b, "asof_join")?;
     }
     Ok(())
 }
@@ -54,6 +59,8 @@ pub enum AsofStrategy {
     Backward,
     /// selects the first row in the right DataFrame whose ‘on’ key is greater than or equal to the left’s key.
     Forward,
+    /// selects the right in the right DataFrame whose 'on' key is nearest to the left's key.
+    Nearest,
 }
 
 impl<T> ChunkedArray<T>
@@ -96,6 +103,9 @@ where
                     )
                 }
             },
+            AsofStrategy::Nearest => {
+                join_asof_nearest(ca.cont_slice().unwrap(), other.cont_slice().unwrap())
+            }
         };
         Ok(out)
     }
