@@ -1,3 +1,4 @@
+mod args;
 pub(crate) mod multiple_keys;
 pub(super) mod single_keys;
 mod single_keys_dispatch;
@@ -7,11 +8,13 @@ mod single_keys_outer;
 #[cfg(feature = "semi_anti_join")]
 mod single_keys_semi_anti;
 pub(super) mod sort_merge;
+mod zip_outer;
 
 use std::fmt::{Debug, Display, Formatter};
 use std::hash::{BuildHasher, Hash, Hasher};
 
 use ahash::RandomState;
+pub use args::*;
 #[cfg(feature = "chunked_ids")]
 use arrow::Either;
 use hashbrown::hash_map::{Entry, RawEntryMut};
@@ -29,8 +32,8 @@ use single_keys_outer::*;
 #[cfg(feature = "semi_anti_join")]
 use single_keys_semi_anti::*;
 pub use sort_merge::*;
+pub(crate) use zip_outer::*;
 
-#[cfg(feature = "private")]
 pub use self::multiple_keys::private_left_join_multiple_keys;
 use crate::datatypes::PlHashMap;
 use crate::frame::groupby::hashing::HASHMAP_INIT_SIZE;
@@ -49,23 +52,7 @@ use crate::prelude::*;
 use crate::utils::{_set_partition_size, slice_slice, split_ca};
 use crate::POOL;
 
-pub type LeftJoinIds = (JoinIds, JoinOptIds);
-
-#[cfg(feature = "chunked_ids")]
-pub(super) type JoinIds = Either<Vec<IdxSize>, Vec<ChunkId>>;
-#[cfg(feature = "chunked_ids")]
-pub type JoinOptIds = Either<Vec<Option<IdxSize>>, Vec<Option<ChunkId>>>;
-
-#[cfg(not(feature = "chunked_ids"))]
-pub type JoinOptIds = Vec<Option<IdxSize>>;
-
-#[cfg(not(feature = "chunked_ids"))]
-pub type JoinIds = Vec<IdxSize>;
-
-/// [ChunkIdx, DfIdx]
-pub type ChunkId = [IdxSize; 2];
-
-pub fn default_join_ids() -> JoinOptIds {
+pub fn default_join_ids() -> ChunkJoinOptIds {
     #[cfg(feature = "chunked_ids")]
     {
         Either::Left(vec![])
@@ -97,6 +84,7 @@ macro_rules! det_hash_prone_order {
 pub(super) use det_hash_prone_order;
 #[cfg(feature = "performant")]
 use polars_arrow::conversion::primitive_to_vec;
+use polars_utils::hash_to_partition;
 
 use crate::series::IsSorted;
 
@@ -114,57 +102,13 @@ pub fn _check_categorical_src(l: &DataType, r: &DataType) -> PolarsResult<()> {
     Ok(())
 }
 
-#[derive(Clone, PartialEq, Eq)]
-#[cfg_attr(feature = "serde", derive(Serialize, Deserialize))]
-pub enum JoinType {
-    Left,
-    Inner,
-    Outer,
-    #[cfg(feature = "asof_join")]
-    AsOf(AsOfOptions),
-    Cross,
-    #[cfg(feature = "semi_anti_join")]
-    Semi,
-    #[cfg(feature = "semi_anti_join")]
-    Anti,
-}
-
-impl Display for JoinType {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        use JoinType::*;
-        let val = match self {
-            Left => "LEFT",
-            Inner => "INNER",
-            Outer => "OUTER",
-            #[cfg(feature = "asof_join")]
-            AsOf(_) => "ASOF",
-            Cross => "CROSS",
-            #[cfg(feature = "semi_anti_join")]
-            Semi => "SEMI",
-            #[cfg(feature = "semi_anti_join")]
-            Anti => "ANTI",
-        };
-        write!(f, "{val}")
-    }
-}
-
-impl Debug for JoinType {
-    fn fmt(&self, f: &mut Formatter<'_>) -> std::fmt::Result {
-        write!(f, "{self}")
-    }
-}
-
 pub(crate) unsafe fn get_hash_tbl_threaded_join_partitioned<Item>(
     h: u64,
     hash_tables: &[Item],
     len: u64,
 ) -> &Item {
-    for i in 0..len {
-        if this_partition(h, i, len) {
-            return hash_tables.get_unchecked(i as usize);
-        }
-    }
-    unreachable!()
+    let i = hash_to_partition(h, len as usize);
+    hash_tables.get_unchecked(i)
 }
 
 #[allow(clippy::type_complexity)]
@@ -173,119 +117,8 @@ unsafe fn get_hash_tbl_threaded_join_mut_partitioned<T, H>(
     hash_tables: &mut [HashMap<T, (bool, Vec<IdxSize>), H>],
     len: u64,
 ) -> &mut HashMap<T, (bool, Vec<IdxSize>), H> {
-    for i in 0..len {
-        if this_partition(h, i, len) {
-            return hash_tables.get_unchecked_mut(i as usize);
-        }
-    }
-    unreachable!()
-}
-
-pub trait ZipOuterJoinColumn {
-    fn zip_outer_join_column(
-        &self,
-        _right_column: &Series,
-        _opt_join_tuples: &[(Option<IdxSize>, Option<IdxSize>)],
-    ) -> Series {
-        unimplemented!()
-    }
-}
-
-impl<T> ZipOuterJoinColumn for ChunkedArray<T>
-where
-    T: PolarsIntegerType,
-    ChunkedArray<T>: IntoSeries,
-{
-    fn zip_outer_join_column(
-        &self,
-        right_column: &Series,
-        opt_join_tuples: &[(Option<IdxSize>, Option<IdxSize>)],
-    ) -> Series {
-        let right_ca = self.unpack_series_matching_type(right_column).unwrap();
-
-        let left_rand_access = self.take_rand();
-        let right_rand_access = right_ca.take_rand();
-
-        opt_join_tuples
-            .iter()
-            .map(|(opt_left_idx, opt_right_idx)| {
-                if let Some(left_idx) = opt_left_idx {
-                    unsafe { left_rand_access.get_unchecked(*left_idx as usize) }
-                } else {
-                    unsafe {
-                        let right_idx = opt_right_idx.unwrap_unchecked();
-                        right_rand_access.get_unchecked(right_idx as usize)
-                    }
-                }
-            })
-            .collect_trusted::<ChunkedArray<T>>()
-            .into_series()
-    }
-}
-
-macro_rules! impl_zip_outer_join {
-    ($chunkedtype:ident) => {
-        impl ZipOuterJoinColumn for $chunkedtype {
-            fn zip_outer_join_column(
-                &self,
-                right_column: &Series,
-                opt_join_tuples: &[(Option<IdxSize>, Option<IdxSize>)],
-            ) -> Series {
-                let right_ca = self.unpack_series_matching_type(right_column).unwrap();
-
-                let left_rand_access = self.take_rand();
-                let right_rand_access = right_ca.take_rand();
-
-                opt_join_tuples
-                    .iter()
-                    .map(|(opt_left_idx, opt_right_idx)| {
-                        if let Some(left_idx) = opt_left_idx {
-                            unsafe { left_rand_access.get_unchecked(*left_idx as usize) }
-                        } else {
-                            unsafe {
-                                let right_idx = opt_right_idx.unwrap_unchecked();
-                                right_rand_access.get_unchecked(right_idx as usize)
-                            }
-                        }
-                    })
-                    .collect::<$chunkedtype>()
-                    .into_series()
-            }
-        }
-    };
-}
-impl_zip_outer_join!(BooleanChunked);
-impl_zip_outer_join!(Utf8Chunked);
-impl_zip_outer_join!(BinaryChunked);
-
-impl ZipOuterJoinColumn for Float32Chunked {
-    fn zip_outer_join_column(
-        &self,
-        right_column: &Series,
-        opt_join_tuples: &[(Option<IdxSize>, Option<IdxSize>)],
-    ) -> Series {
-        self.apply_as_ints(|s| {
-            s.zip_outer_join_column(
-                &right_column.bit_repr_small().into_series(),
-                opt_join_tuples,
-            )
-        })
-    }
-}
-
-impl ZipOuterJoinColumn for Float64Chunked {
-    fn zip_outer_join_column(
-        &self,
-        right_column: &Series,
-        opt_join_tuples: &[(Option<IdxSize>, Option<IdxSize>)],
-    ) -> Series {
-        self.apply_as_ints(|s| {
-            s.zip_outer_join_column(
-                &right_column.bit_repr_large().into_series(),
-                opt_join_tuples,
-            )
-        })
-    }
+    let i = hash_to_partition(h, len as usize);
+    hash_tables.get_unchecked_mut(i)
 }
 
 pub fn _join_suffix_name(name: &str, suffix: &str) -> String {
@@ -368,13 +201,12 @@ impl DataFrame {
         &self,
         ids: LeftJoinIds,
         other: &DataFrame,
-        suffix: Option<String>,
-        slice: Option<(i64, usize)>,
+        args: JoinArgs,
     ) -> PolarsResult<DataFrame> {
         let (left_idx, right_idx) = ids;
         let materialize_left = || {
             let mut left_idx = &*left_idx;
-            if let Some((offset, len)) = slice {
+            if let Some((offset, len)) = args.slice {
                 left_idx = slice_slice(left_idx, offset, len);
             }
             unsafe { self._create_left_df_from_slice(left_idx, true, true) }
@@ -382,7 +214,7 @@ impl DataFrame {
 
         let materialize_right = || {
             let mut right_idx = &*right_idx;
-            if let Some((offset, len)) = slice {
+            if let Some((offset, len)) = args.slice {
                 right_idx = slice_slice(right_idx, offset, len);
             }
             unsafe {
@@ -393,7 +225,7 @@ impl DataFrame {
         };
         let (df_left, df_right) = POOL.join(materialize_left, materialize_right);
 
-        _finish_join(df_left, df_right, suffix.as_deref())
+        _finish_join(df_left, df_right, args.suffix.as_deref())
     }
 
     #[cfg(feature = "chunked_ids")]
@@ -401,19 +233,20 @@ impl DataFrame {
         &self,
         ids: LeftJoinIds,
         other: &DataFrame,
-        suffix: Option<String>,
-        slice: Option<(i64, usize)>,
+        args: JoinArgs,
     ) -> PolarsResult<DataFrame> {
+        let suffix = &args.suffix;
+        let slice = args.slice;
         let (left_idx, right_idx) = ids;
         let materialize_left = || match left_idx {
-            JoinIds::Left(left_idx) => {
+            ChunkJoinIds::Left(left_idx) => {
                 let mut left_idx = &*left_idx;
                 if let Some((offset, len)) = slice {
                     left_idx = slice_slice(left_idx, offset, len);
                 }
                 unsafe { self._create_left_df_from_slice(left_idx, true, true) }
             }
-            JoinIds::Right(left_idx) => {
+            ChunkJoinIds::Right(left_idx) => {
                 let mut left_idx = &*left_idx;
                 if let Some((offset, len)) = slice {
                     left_idx = slice_slice(left_idx, offset, len);
@@ -423,7 +256,7 @@ impl DataFrame {
         };
 
         let materialize_right = || match right_idx {
-            JoinOptIds::Left(right_idx) => {
+            ChunkJoinOptIds::Left(right_idx) => {
                 let mut right_idx = &*right_idx;
                 if let Some((offset, len)) = slice {
                     right_idx = slice_slice(right_idx, offset, len);
@@ -434,7 +267,7 @@ impl DataFrame {
                     )
                 }
             }
-            JoinOptIds::Right(right_idx) => {
+            ChunkJoinOptIds::Right(right_idx) => {
                 let mut right_idx = &*right_idx;
                 if let Some((offset, len)) = slice {
                     right_idx = slice_slice(right_idx, offset, len);
@@ -452,8 +285,7 @@ impl DataFrame {
         other: &DataFrame,
         s_left: &Series,
         s_right: &Series,
-        suffix: Option<String>,
-        slice: Option<(i64, usize)>,
+        args: JoinArgs,
         verbose: bool,
     ) -> PolarsResult<DataFrame> {
         #[cfg(feature = "dtype-categorical")]
@@ -472,8 +304,8 @@ impl DataFrame {
             right.as_single_chunk_par();
             s_right = s_right.rechunk();
         }
-        let ids = sort_or_hash_left(&s_left, &s_right, verbose);
-        left._finish_left_join(ids, &right.drop(s_right.name()).unwrap(), suffix, slice)
+        let ids = sort_or_hash_left(&s_left, &s_right, verbose, args.validation)?;
+        left._finish_left_join(ids, &right.drop(s_right.name()).unwrap(), args)
     }
 
     #[cfg(feature = "semi_anti_join")]
@@ -512,8 +344,7 @@ impl DataFrame {
         other: &DataFrame,
         s_left: &Series,
         s_right: &Series,
-        suffix: Option<String>,
-        slice: Option<(i64, usize)>,
+        args: JoinArgs,
     ) -> PolarsResult<DataFrame> {
         #[cfg(feature = "dtype-categorical")]
         _check_categorical_src(s_left.dtype(), s_right.dtype())?;
@@ -522,10 +353,10 @@ impl DataFrame {
         let join_column_index = self.iter().position(|s| s.name() == s_left.name()).unwrap();
 
         // Get the indexes of the joined relations
-        let opt_join_tuples = s_left.hash_join_outer(s_right);
+        let opt_join_tuples = s_left.hash_join_outer(s_right, args.validation)?;
         let mut opt_join_tuples = &*opt_join_tuples;
 
-        if let Some((offset, len)) = slice {
+        if let Some((offset, len)) = args.slice {
             opt_join_tuples = slice_slice(opt_join_tuples, offset, len);
         }
 
@@ -572,6 +403,6 @@ impl DataFrame {
         };
 
         unsafe { df_left.get_columns_mut().insert(join_column_index, s) };
-        _finish_join(df_left, df_right, suffix.as_deref())
+        _finish_join(df_left, df_right, args.suffix.as_deref())
     }
 }
