@@ -2,7 +2,6 @@ use arrow::bitmap::utils::get_bit_unchecked;
 use hashbrown::hash_map::RawEntryMut;
 use hashbrown::HashMap;
 use polars_arrow::utils::CustomIterTools;
-use polars_utils::HashSingle;
 use rayon::prelude::*;
 use xxhash_rust::xxh3::xxh3_64_with_seed;
 
@@ -164,23 +163,26 @@ impl VecHash for Utf8Chunked {
     }
 }
 
+// used in polars-pipe
+pub fn _hash_binary_array(arr: &BinaryArray<i64>, random_state: RandomState, buf: &mut Vec<u64>) {
+    let null_h = get_null_hash_value(random_state);
+    if arr.null_count() == 0 {
+        // use the null_hash as seed to get a hash determined by `random_state` that is passed
+        buf.extend(arr.values_iter().map(|v| xxh3_64_with_seed(v, null_h)))
+    } else {
+        buf.extend(arr.into_iter().map(|opt_v| match opt_v {
+            Some(v) => xxh3_64_with_seed(v, null_h),
+            None => null_h,
+        }))
+    }
+}
+
 impl VecHash for BinaryChunked {
     fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
         buf.clear();
         buf.reserve(self.len());
-        let null_h = get_null_hash_value(random_state);
-
-        self.downcast_iter().for_each(|arr| {
-            if arr.null_count() == 0 {
-                // simply use the null_hash as seed to get a hash determined by `random_state` that is passed
-                buf.extend(arr.values_iter().map(|v| xxh3_64_with_seed(v, null_h)))
-            } else {
-                buf.extend(arr.into_iter().map(|opt_v| match opt_v {
-                    Some(v) => xxh3_64_with_seed(v, null_h),
-                    None => null_h,
-                }))
-            }
-        });
+        self.downcast_iter()
+            .for_each(|arr| _hash_binary_array(arr, random_state.clone(), buf));
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
@@ -222,8 +224,8 @@ impl VecHash for BooleanChunked {
     fn vec_hash(&self, random_state: RandomState, buf: &mut Vec<u64>) {
         buf.clear();
         buf.reserve(self.len());
-        let true_h = random_state.hash_single(true);
-        let false_h = random_state.hash_single(false);
+        let true_h = random_state.hash_one(true);
+        let false_h = random_state.hash_one(false);
         let null_h = get_null_hash_value(random_state);
         self.downcast_iter().for_each(|arr| {
             if arr.null_count() == 0 {
@@ -239,8 +241,8 @@ impl VecHash for BooleanChunked {
     }
 
     fn vec_hash_combine(&self, random_state: RandomState, hashes: &mut [u64]) {
-        let true_h = random_state.hash_single(true);
-        let false_h = random_state.hash_single(false);
+        let true_h = random_state.hash_one(true);
+        let false_h = random_state.hash_one(false);
         let null_h = get_null_hash_value(random_state);
 
         let mut offset = 0;
